[llvm-commits] [llvm] r112158 - /llvm/trunk/lib/Transforms/Utils/ValueMapper.cpp

Daniel Dunbar daniel at zuster.org
Wed Aug 25 20:48:12 PDT 2010


Author: ddunbar
Date: Wed Aug 25 22:48:11 2010
New Revision: 112158

URL: http://llvm.org/viewvc/llvm-project?rev=112158&view=rev
Log:
Revert r111922, "MapValue support for MDNodes. This is similar to r109117,
except ...", it is causing *massive* performance regressions when building Clang
with itself (-O3 -g).

Modified:
    llvm/trunk/lib/Transforms/Utils/ValueMapper.cpp

Modified: llvm/trunk/lib/Transforms/Utils/ValueMapper.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/lib/Transforms/Utils/ValueMapper.cpp?rev=112158&r1=112157&r2=112158&view=diff
==============================================================================
--- llvm/trunk/lib/Transforms/Utils/ValueMapper.cpp (original)
+++ llvm/trunk/lib/Transforms/Utils/ValueMapper.cpp Wed Aug 25 22:48:11 2010
@@ -27,36 +27,17 @@
   // NOTE: VMSlot can be invalidated by any reference to VM, which can grow the
   // DenseMap.  This includes any recursive calls to MapValue.
 
-  // Global values do not need to be seeded into the VM if they
-  // are using the identity mapping.
-  if (isa<GlobalValue>(V) || isa<InlineAsm>(V) || isa<MDString>(V))
+  // Global values and non-function-local metadata do not need to be seeded into
+  // the VM if they are using the identity mapping.
+  if (isa<GlobalValue>(V) || isa<InlineAsm>(V) || isa<MDString>(V) ||
+      (isa<MDNode>(V) && !cast<MDNode>(V)->isFunctionLocal()))
     return VMSlot = const_cast<Value*>(V);
 
   if (const MDNode *MD = dyn_cast<MDNode>(V)) {
-    // Start by assuming that we'll use the identity mapping.
-    VMSlot = const_cast<Value*>(V);
-
-    // Check all operands to see if any need to be remapped.
-    for (unsigned i = 0, e = MD->getNumOperands(); i != e; ++i) {
-      Value *OP = MD->getOperand(i);
-      if (!OP || MapValue(OP, VM) == OP) continue;
-
-      // Ok, at least one operand needs remapping.
-      MDNode *Dummy = MDNode::getTemporary(V->getContext(), 0, 0);
-      VM[V] = Dummy;
-      SmallVector<Value*, 4> Elts;
-      Elts.reserve(MD->getNumOperands());
-      for (i = 0; i != e; ++i)
-        Elts.push_back(MD->getOperand(i) ? 
-                       MapValue(MD->getOperand(i), VM) : 0);
-      MDNode *NewMD = MDNode::get(V->getContext(), Elts.data(), Elts.size());
-      Dummy->replaceAllUsesWith(NewMD);
-      MDNode::deleteTemporary(Dummy);
-      return VM[V] = NewMD;
-    }
-
-    // No operands needed remapping; keep the identity map.
-    return const_cast<Value*>(V);
+    SmallVector<Value*, 4> Elts;
+    for (unsigned i = 0, e = MD->getNumOperands(); i != e; ++i)
+      Elts.push_back(MD->getOperand(i) ? MapValue(MD->getOperand(i), VM) : 0);
+    return VM[V] = MDNode::get(V->getContext(), Elts.data(), Elts.size());
   }
 
   Constant *C = const_cast<Constant*>(dyn_cast<Constant>(V));





More information about the llvm-commits mailing list